using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine; 
using UnityEngine.Events;

namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
            ModAPI.RegisterLiquid(SubstanceASyringe.SubstanceASerum.ID, new SubstanceASyringe.SubstanceASerum());
            
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Substance A",
                    DescriptionOverride = "A deep briny sample of bog-water.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("substancea.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<SubstanceASyringe>();
                    }
                }
            );
		ModAPI.RegisterLiquid(SubstanceBSyringe.SubstanceBSerum.ID, new SubstanceBSyringe.SubstanceBSerum());
            
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Substance B",
                    DescriptionOverride = "A thick, pungent vapor found in the colon of an infernal beast.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("substanceb.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<SubstanceBSyringe>();
                    }
                }
            );
	ModAPI.RegisterLiquid(SubstanceCSyringe.SubstanceCSerum.ID, new SubstanceCSyringe.SubstanceCSerum());
            
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Substance C",
                    DescriptionOverride = "A shimmering mass of tiny silver pellets.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("substancec.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<SubstanceCSyringe>();
                    }
                }
            );
	ModAPI.RegisterLiquid(SubstanceDSyringe.SubstanceDSerum.ID, new SubstanceDSyringe.SubstanceDSerum());
            
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Substance D",
                    DescriptionOverride = "A stone that cracks and shifts.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("substanced.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<SubstanceDSyringe>();
                    }
                }
            );
            ModAPI.RegisterLiquid(SubstanceESyringe.SubstanceESerum.ID, new SubstanceESyringe.SubstanceESerum());
            
            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),
                    NameOverride = "Substance E",
                    DescriptionOverride = "Found in a mountaintop shrine, it resists change.",
                    CategoryOverride = ModAPI.FindCategory("Biohazard"),
                    ThumbnailOverride = ModAPI.LoadSprite("substancee.png"),
                    AfterSpawn = (Instance) =>
                    {
                        UnityEngine.Object.Destroy(Instance.GetComponent<SyringeBehaviour>());
                        Instance.GetOrAddComponent<SubstanceESyringe>();
                    }
                }
            );
        }
    }

    public class SubstanceASyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => SubstanceASerum.ID;

        public class SubstanceASerum : Liquid
        {
            public const string ID = "SUBSTANCE A";

            public SubstanceASerum()
            {
                Color = new UnityEngine.Color(0.24f, 0.38f, 0.36f);
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
                limb.Vitality = 1;
		limb.Health = 1;
		limb.BreakingThreshold = 1;
            }

            public override void OnUpdate(BloodContainer container)
	{
            if (container is CirculationBehaviour circ && circ.Limb.SpeciesIdentity != Species.Android)
            {
                circ.Limb.PhysicalBehaviour.BurnProgress *= 0.5f;
            }
	}
            public override void OnEnterContainer(BloodContainer container) {}
            public override void OnExitContainer(BloodContainer container) {}

        }
    }

    public class SubstanceBSyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => SubstanceBSerum.ID;

        public class SubstanceBSerum : Liquid
        {
            public const string ID = "SUBSTANCE B";

            public SubstanceBSerum()
            {
                Color = new UnityEngine.Color(0.64f, 0.83f, 0.86f);
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
		limb.BodyTemperature = -200f;
		limb.Numbness = 0.1f;
            }

            public override void OnUpdate(BloodContainer container)
	{
            // if this container is a CirculationBehaviour, meaning it is inside a limb && its species is not Android
            if (container is CirculationBehaviour circ && circ.Limb.SpeciesIdentity != Species.Android)
            {
                // half the RottenProgress value
                circ.Limb.SkinMaterialHandler.AcidProgress *= 0.75f;
            }
	}
            public override void OnEnterContainer(BloodContainer container) {}
            public override void OnExitContainer(BloodContainer container) {}
        }
    }

 	public class SubstanceCSyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => SubstanceCSerum.ID;

        public class SubstanceCSerum : Liquid
        {
            public const string ID = "SUBSTANCE C";

            public SubstanceCSerum()
            {
                Color = new UnityEngine.Color(0.66f, 0.59f, 0.59f);
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
		limb.IsAndroid = true;
		limb.Vitality = 10f;
		limb.Health = 75f;
		limb.BloodLiquidType = "SUBSTANCE C";
		limb.SkinMaterialHandler.ShouldRot = true;
		limb.SkinMaterialHandler.RottenProgress += 0.1f;
            }

            public override void OnUpdate(BloodContainer container) 
        {
            if (container is CirculationBehaviour circ && circ.Limb.SpeciesIdentity != Species.Android)
            {
                circ.Limb.SkinMaterialHandler.RottenProgress += 0.075f;
                circ.Limb.SkinMaterialHandler.AcidProgress += 0.02f;
                circ.Limb.PhysicalBehaviour.BurnIntensity *= 0.5f;
            }
        }
            public override void OnEnterContainer(BloodContainer container) {}
            public override void OnExitContainer(BloodContainer container) {}
        }
    }

    public class SubstanceDSyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => SubstanceDSerum.ID;

        public class SubstanceDSerum : Liquid
        {
            public const string ID = "SUBSTANCE D";

            public SubstanceDSerum()
            {
                Color = new UnityEngine.Color(0.42f, 0.28f, 0.23f);
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
                limb.FreezingTemperature = -210f;
		limb.SkinMaterialHandler.MinFreezeTemperature = -210;
            }

            public override void OnUpdate(BloodContainer container)
        {
            // if this container is a CirculationBehaviour, meaning it is inside a limb && its species is not Android
            if (container is CirculationBehaviour circ && circ.Limb.SpeciesIdentity != Species.Android)
            {
                circ.Limb.PhysicalBehaviour.Charge += 0.3f;
                circ.Limb.SkinMaterialHandler.RottenProgress *= 0.8f;
            }
        }
            public override void OnEnterContainer(BloodContainer container) {}
            public override void OnExitContainer(BloodContainer container) {}

        }
    }

    public class SubstanceESyringe : SyringeBehaviour
    {
        public override string GetLiquidID() => SubstanceESerum.ID;

        public class SubstanceESerum : Liquid
        {
            public const string ID = "SUBSTANCE E";

            public SubstanceESerum()
            {
                Color = new UnityEngine.Color(0.1f, 0.99f, 0.92f);
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
		limb.PhysicalBehaviour.ForceNoCharge = true;
		limb.RegenerationSpeed = 3;
		limb.InternalTemperature = 175f;
		limb.PhysicalBehaviour.Temperature = 175f;
            }

            public override void OnUpdate(BloodContainer container) {}
            public override void OnEnterContainer(BloodContainer container) {}
            public override void OnExitContainer(BloodContainer container) {}

        }
    }
}

